// ---------------------------------------------------------------------------
// Code for soial-distancing emoti-mask by Ayesha Iftiqhar
// ---------------------------------------------------------------------------

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h>
#endif

// define pin numbers for Ultrasonic Sensor and NeoPixels
#define SensorIn  10      //Digital IO pin connected to the ultrasonic sensor
#define PIXEL_PIN    6    // Digital IO pin connected to the NeoPixels

#define NUMPIXELS    14   //Number of neopixels connected

// delay NeoPixel
int delayval = 500; // delay for half a second

// defines variables
int arraysize = 9; //Needs to be an odd number to find the Median
float array[] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
long RawData;
float SocialDistance = 0.0;


Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIXEL_PIN, NEO_GRB + NEO_KHZ800);


void setup() {

  //Initialize Pixel
  pixels.begin();
  pixels.show();
  pixels.setBrightness(50);

  pinMode (SensorIn, INPUT); // sets the Sensor pin as an Input
  // Serial.begin(9600); //start serial communication (for debugging)
}

void sensorRead() {
  for (int i = 0; i < arraysize; i++)
  {
    RawData = pulseIn(SensorIn, HIGH);
    array[i] = (RawData * 0.0135) / 12; //Convert distance in feet
    delay(10);
  }
}

// Arrange the array of inputs in ascending order
void array_arrangment(float *a, int n) {
  for (int i = 1; i < n; ++i)
  {
    float j = a[i];
    int k;
    for (k = i - 1; (k >= 0) && (j < a[k]); k--)
    {
      a[k + 1] = a[k];
    }
    a[k + 1] = j;
  }
}

// Find mode of the array or median of the array, when there is no mode
float filter(float *a, int n) {
  int i = 0;
  int count = 0;
  int maxCount = 0;
  float filter = 0.0;
  int median;
  int prevCount = 0;

  while (i < (n - 1)) {
    prevCount = count;
    count = 0;
    while (a[i] == a[i + 1]) {
      count++;
      i++;
    }
    if (count > prevCount && count > maxCount) {
      filter = a[i];
      maxCount = count;
      median = 0;
    }
    if (count == 0) {
      i++;
    }
    if (count == maxCount) { //If the dataset has 2 or more modes.
      median = 1;
    }
    if (filter == 0 || median == 1) { //Return the median if there is no mode.
      filter = a[(n / 2)];
    }
    return filter;

  }
}

void loop() {

  sensorRead();
  array_arrangment(array, arraysize);
  SocialDistance = filter(array, arraysize);
  // Serial.print("SocialDistance:"); // For debugging
  // Serial.println(SocialDistance);  // For debugging
  delay(100);


  //If someone gets closer than 3ft to you, show a shocked face (How dare you!)
  if (SocialDistance <= 3) {
    shocked_face();
    // wait a bit
    delay(100);
  }


  //If someone gets closer than 5ft to you but still more than 3ft away, show a sad face (why you wanna gimme you're germs, I thought we were friends!)
  if ((SocialDistance <= 5) && (SocialDistance > 3) ) {
    sad_face();
    // wait a bit
    delay(100);
  }


  //If someone gets closer than 5.9ft to you but still more than a 5ft away, show a straight face (like bruh, wtf!)
  if ((SocialDistance <= 5.9) && (SocialDistance > 5) ) {
    straight_face();
    // wait a bit
    delay(100);
  }


  //If someone is at around 6ft, show a smiley face (cool, don't get any closer now!)
  if ((SocialDistance <= 6.5) && (SocialDistance > 5.9) ) {
    smiley_face();
    // wait a bit
    delay(100);
  }


  //If someone is beyonf 6ft, show a grin (life is great, when you're social distancing!)
  else if (SocialDistance > 6.5)
  {
    grin_face();
    // wait a bit
    delay(100);
  }
  delay(100);  // wait 100 us before next read


}

/*setPixelColor(uint16_t n, uint8_t r, uint8_t g, uint8_t b)*/
void shocked_face (void) {
  pixels.clear();
  pixels.setPixelColor(2, pixels.Color(0, 255, 0));
  pixels.setPixelColor(3, pixels.Color(0, 255, 0));
  pixels.setPixelColor(6, pixels.Color(0, 255, 0));
  pixels.setPixelColor(9, pixels.Color(0, 255, 0));
  pixels.setPixelColor(11, pixels.Color(0, 255, 0));
  pixels.setPixelColor(12, pixels.Color(0, 255, 0));
  pixels.show();
  //Serial.println("Shock");

}

void sad_face (void) {
  pixels.clear();
  pixels.setPixelColor(2, pixels.Color(0, 0, 255));
  pixels.setPixelColor(3, pixels.Color(0, 0, 255));
  pixels.setPixelColor(6, pixels.Color(0, 0, 255));
  pixels.setPixelColor(9, pixels.Color(0, 0, 255));
  pixels.setPixelColor(10, pixels.Color(0, 0, 255));
  pixels.setPixelColor(13, pixels.Color(0, 0, 255));
  pixels.show();  // This sends the updated pixel color to the hardware.
  //Serial.println("Sad");
}

void straight_face (void) {
  pixels.clear();
  pixels.setPixelColor(6, pixels.Color(255, 255, 255));
  pixels.setPixelColor(7, pixels.Color(255, 255, 255));
  pixels.setPixelColor(8, pixels.Color(255, 255, 255));
  pixels.setPixelColor(9, pixels.Color(255, 255, 255));
  pixels.show();  // This sends the updated pixel color to the hardware.
  // Serial.println("Blank");
}

void smiley_face (void) {
  pixels.clear();
  pixels.setPixelColor(0, pixels.Color(255, 0, 0));
  pixels.setPixelColor(5, pixels.Color(255, 0, 0));
  pixels.setPixelColor(6, pixels.Color(255, 0, 0));
  pixels.setPixelColor(9, pixels.Color(255, 0, 0));
  pixels.setPixelColor(11, pixels.Color(255, 0, 0));
  pixels.setPixelColor(12, pixels.Color(255, 0, 0));
  pixels.show();  // This sends the updated pixel color to the hardware.
  //Serial.println("Smile");
}

void grin_face (void) {
  pixels.clear();
  pixels.setPixelColor(0, pixels.Color(255, 0, 0));
  pixels.setPixelColor(1, pixels.Color(255, 0, 0));
  pixels.setPixelColor(2, pixels.Color(255, 0, 0));
  pixels.setPixelColor(3, pixels.Color(255, 0, 0));
  pixels.setPixelColor(4, pixels.Color(255, 0, 0));
  pixels.setPixelColor(6, pixels.Color(255, 0, 0));
  pixels.setPixelColor(5, pixels.Color(255, 0, 0));
  pixels.setPixelColor(7, pixels.Color(255, 0, 0));
  pixels.setPixelColor(9, pixels.Color(255, 0, 0));
  pixels.setPixelColor(11, pixels.Color(255, 0, 0));
  pixels.setPixelColor(12, pixels.Color(255, 0, 0));
  pixels.setPixelColor(8, pixels.Color(255, 255, 255));
  pixels.setPixelColor(7, pixels.Color(255, 255, 255));
  pixels.show();  // This sends the updated pixel color to the hardware.
  //Serial.println("Grin");
}
